package cc.fascinated.bat.features.base.commands.server.channel;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.Channel;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Nick (okNick)
 */
@Component
@CommandInfo(name = "removetopic", description = "Remove the topic of a channel", requiredPermissions = Permission.MANAGE_CHANNEL)
public class RemoveTopicSubCommand extends BatSubCommand {
    public RemoveTopicSubCommand() {
        super.addOption(OptionType.CHANNEL, "channel", "The channel to remove the topic of", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        Channel target = event.getOption("channel") == null ? channel : event.getOption("channel").getAsChannel();
        if (!(target instanceof TextChannel textChannel)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> is not a text channel!".formatted(target.getId()))
                            .build())
                    .queue();
            return;
        }

        if (textChannel.getTopic() == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> does not have a topic!".formatted(textChannel.getId()))
                            .build())
                    .queue();
            return;
        }

        textChannel.getManager().setTopic(null).queue();
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully removed the topic of <#%s>".formatted(textChannel.getId()))
                .build()
        ).queue();
    }
}