package cc.fascinated.bat.features.logging;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor @Getter
public enum LogType {
    /**
     * Message Events
     */
    MESSAGE_DELETE(LogCategory.MESSAGE, "Message Delete"),
    MESSAGE_EDIT(LogCategory.MESSAGE,"Message Edit"),

    /**
     * Member Events
     */
    MEMBER_JOIN(LogCategory.MEMBER, "Member Join"),
    MEMBER_LEAVE(LogCategory.MEMBER, "Member Leave"),
    MEMBER_NICKNAME_UPDATE(LogCategory.MEMBER, "Member Nickname Update"),
    MEMBER_GLOBAL_NAME_UPDATE(LogCategory.MEMBER, "Member Global Name Update"),
    MEMBER_USERNAME_UPDATE(LogCategory.MEMBER, "Member Username Update"),
    MEMBER_AVATAR_UPDATE(LogCategory.MEMBER, "Member Avatar Update"),
    MEMBER_ROLE_UPDATE(LogCategory.MEMBER, "Member Role Update"),
    MEMBER_BAN(LogCategory.MEMBER, "Member Ban"),
    MEMBER_UNBAN(LogCategory.MEMBER, "Member Unban"),
    MEMBER_TIMEOUT(LogCategory.MEMBER, "Member Timeout"),

    /**
     * Channel Events
     */
    CHANNEL_CREATE(LogCategory.CHANNEL, "Channel Create"),
    CHANNEL_DELETE(LogCategory.CHANNEL, "Channel Delete"),
    VOICE_CHANNEL_JOIN(LogCategory.CHANNEL, "Voice Channel Join"),
    VOICE_CHANNEL_LEAVE(LogCategory.CHANNEL, "Voice Channel Leave");

    /**
     * The category of the log type
     */
    private final LogCategory category;

    /**
     * The name of the log type
     */
    private final String name;

    /**
     * Gets the log type by the name
     *
     * @param name - the name
     * @return the log type, or null if it doesn't exist
     */
    public static LogType getLogType(String name) {
        for (LogType logType : values()) {
            if (logType.getName().equalsIgnoreCase(name)) {
                return logType;
            }
        }
        return null;
    }

    /**
     * Gets the log types by the category
     *
     * @param category - the category
     * @return the log types
     */
    public static List<LogType> getLogTypesByCategory(String category) {
        List<LogType> logTypes = new ArrayList<>();
        for (LogType logType : values()) {
            if (logType.getCategory().getName().equalsIgnoreCase(category)) {
                logTypes.add(logType);
            }
        }
        return logTypes;
    }
}
