package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:list.sub")
@CommandInfo(name = "list", description = "See all the log types and their channels")
public class ListSubCommand extends BatSubCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        LogProfile profile = guild.getLogProfile();
        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Log Channels");
        description.appendLine("""
                Set the log channel for:
                - A specific event, use `/logs set <event> <channel>`
                - A specific category by using `/logs set <category> <channel>`
                - All log types by using `/logs set all <channel>`
                To remove a log channel, it's the same as setting it,
                but with `/logs remove` instead of `/logs set`""", false);
        description.emptyLine();
        for (int i = 0; i < LogCategory.values().length; i++) {
            LogCategory category = LogCategory.values()[i];
            if (i != 0) {
                description.emptyLine();
            }
            description.appendLine("**__%s__**".formatted(category.getName()), false);
            for (LogType logType : LogType.values()) {
                if (logType.getCategory() == category) {
                    TextChannel logChannel = profile.getLogChannel(logType);
                    description.appendLine("%s: %s".formatted(logType.getName(), logChannel == null ? "Not Set" : logChannel.getAsMention()), true);
                }
            }
        }
        event.replyEmbeds(EmbedUtils.genericEmbed().setDescription(description.build()).build()).queue();
    }
}

