package cc.fascinated.bat.features.namehistory.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.namehistory.TrackedName;
import cc.fascinated.bat.features.namehistory.profile.guild.NameHistoryProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("namehistory:guild.sub")
@CommandInfo(name = "guild", description = "View the guild nickname history of a user")
public class GuildSubCommand extends BatSubCommand {
    private final UserService userService;

    @Autowired
    public GuildSubCommand(@NonNull UserService userService) {
        this.userService = userService;
        super.addOption(OptionType.USER, "user", "The user to view the name history of", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping userOption = event.getOption("user");
        BatUser target = userOption == null ? user : userService.getUser(userOption.getAsUser().getId());
        if (target == null) {
            channel.sendMessage("User not found").queue();
            return;
        }

        NameHistoryProfile profile = guild.getNameHistoryProfile();
        StringBuilder builder = new StringBuilder();
        if (profile.getNameHistory(target).isEmpty()) {
            builder.append("%s has no name history".formatted(target.getDiscordUser().getAsMention()));
        } else {
            for (TrackedName trackedName : profile.getNameHistorySorted(target)) {
                builder.append("%s - <t:%s>\n".formatted(
                        trackedName.getName() == null ? "Removed Nickname" : "`%s`".formatted(trackedName.getName()),
                        trackedName.getChangedDate().toInstant().toEpochMilli()/1000
                ));
            }
        }

        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setAuthor("%s's Nickname History in %s".formatted(target.getName(), guild.getName()))
                .setDescription(builder.toString())
                .build()
        ).queue();
    }
}
