package cc.fascinated.bat.features.namehistory.profile.user;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.features.namehistory.NameHistoryFeature;
import cc.fascinated.bat.features.namehistory.TrackedName;
import com.google.gson.Gson;
import lombok.NoArgsConstructor;
import org.bson.Document;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@NoArgsConstructor
public class NameHistoryProfile extends Serializable {
    /**
     * The name history of the user
     */
    private List<TrackedName> nameHistory;

    /**
     * Gets the name history of the user
     *
     * @return the name history of the user
     */
    public List<TrackedName> getNameHistory() {
        if (this.nameHistory == null) {
            this.nameHistory = new LinkedList<>();
        }
        return nameHistory;
    }

    /**
     * Gets the name history of the user sorted
     *
     * @return the name history of the user sorted
     */
    public List<TrackedName> getNameHistorySorted() {
        return getNameHistory().stream().sorted((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate())).toList();
    }

    /**
     * Adds a name to the name history
     *
     * @param name the name to add
     */
    public void addName(String name) {
        if (this.nameHistory == null) {
            this.nameHistory = new LinkedList<>();
        }
        TrackedName trackedName = new TrackedName();
        trackedName.setName(name);
        trackedName.setChangedDate(new Date());
        this.nameHistory.add(trackedName);
        cleanup();
    }

    /**
     * Cleans up the name history
     * <p>
     * This will remove any names that are not within
     * the size limit of the name history
     * </p>
     */
    private void cleanup() {
        if (this.nameHistory.size() > NameHistoryFeature.NAME_HISTORY_SIZE) {
            List<TrackedName> trackedNames = new ArrayList<>(this.nameHistory);
            trackedNames.sort((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate()));
            this.nameHistory = trackedNames.subList(0, NameHistoryFeature.NAME_HISTORY_SIZE);
        }
    }

    @Override
    public void reset() {
        this.nameHistory = null;
    }

    @Override
    public void load(Document document, Gson gson) {
        this.nameHistory = new LinkedList<>();
        for (Document trackedNameDocument : document.getList("nameHistory", Document.class, new ArrayList<>())) {
            TrackedName trackedName = new TrackedName();
            trackedName.load(trackedNameDocument, gson);

            this.nameHistory.add(trackedName);
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        List<Document> trackedNames = new ArrayList<>();
        for (TrackedName trackedName : this.nameHistory) {
            Document trackedNameDocument = new Document();
            trackedNameDocument.put("name", trackedName.getName());
            trackedNameDocument.put("changedDate", trackedName.getChangedDate());
            trackedNames.add(trackedNameDocument);
        }
        document.put("nameHistory", trackedNames);
        return document;
    }
}
