package cc.fascinated.bat.features.scoresaber.profile.guild;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.service.DiscordService;
import com.google.gson.Gson;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.bson.Document;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
@Setter
@NoArgsConstructor
public class UserScoreFeedProfile extends Serializable {
    /**
     * The channel ID of the score feed
     */
    private String channelId;

    /**
     * The users that are being tracked
     */
    private List<String> trackedUsers;

    /**
     * Gets the tracked users
     *
     * @return the tracked users
     */
    public List<String> getTrackedUsers() {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        return trackedUsers;
    }

    /**
     * Checks if a user is being tracked
     *
     * @param userId the user ID to check
     * @return if the user is being tracked
     */
    public boolean isUserTracked(String userId) {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        return trackedUsers.contains(userId);
    }

    /**
     * Adds a user to be tracked
     *
     * @param userId the user ID to add
     */
    public void addTrackedUser(String userId) {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        trackedUsers.add(userId);
    }

    /**
     * Removes a user from being tracked
     *
     * @param userId the user ID to remove
     */
    public void removeTrackedUser(String userId) {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        trackedUsers.remove(userId);
    }

    /**
     * Gets the channel as a TextChannel
     *
     * @return the channel as a TextChannel
     */
    public TextChannel getTextChannel() {
        return DiscordService.JDA.getTextChannelById(channelId);
    }

    @Override
    public void reset() {
        this.channelId = null;
        this.trackedUsers = null;
    }

    @Override
    public void load(Document document, Gson gson) {
        this.channelId = document.getString("channelId");
        this.trackedUsers = document.getList("trackedUsers", String.class, new ArrayList<>());
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        document.put("channelId", this.channelId);
        document.put("trackedUsers", this.trackedUsers);
        return document;
    }
}
